#!/usr/bin/env python

import gcc

import os, sys

from PyQt4 import QtCore, QtGui

# --------------------------------------------------------------------------

icon_dir = "~/temp/monodevelop/monodevelop-2.8.8.4/src/core/MonoDevelop.Ide/icons"
# icon_dir = "~/temp/monodevelop/monodevelop-4.0.14/src/core/MonoDevelop.Ide/icons"
# icon_dir = "~/temp/monodevelop/monodevelop-5.7/src/core/MonoDevelop.Ide/icons/light"
icon_dir = os.path.expanduser (icon_dir)

QtGui.QIcon.setThemeName ("oxygen")

# --------------------------------------------------------------------------

def findIcon (icon_name):
    global icon_dir
    file_name = os.path.join (icon_dir, icon_name + ".png")
    if os.path.isfile (file_name) :
       icon = QtGui.QIcon (os.path.join (icon_dir, icon_name + ".png"))
    else :
       icon = QtGui.QIcon.fromTheme (icon_name)
    return icon

def findIcons (icon_name1, icon_name2):
    icon = findIcon (icon_name1)
    if icon.isNull () and icon_name2:
       icon = findIcon (icon_name2)
    return icon

class TreeItem (QtGui.QTreeWidgetItem):
    def __init__ (self, parent, text):
        QtGui.QTreeWidgetItem.__init__ (self, parent)
        self.setText (0, text)

    def addIcon (self, icon_name):
        self.setIcon (0, findIcon (icon_name))

    def addIcons (self, icon_name1, icon_name2):
        self.setIcon (0, findIcons (icon_name1, icon_name2))

class TreeView (QtGui.QTreeWidget):
    def __init__ (self, parent=None):
        QtGui.QTreeWidget.__init__ (self, parent)
        self.setColumnCount (1)
        self.header ().hide ()
        # self.setRootIsDecorated (True)
        # self.setSelectionMode (QtGui.QAbstractItemView.ExtendedSelection)
        self.setEditTriggers (QtGui.QAbstractItemView.SelectedClicked)
        self.setIndentation (8);

# --------------------------------------------------------------------------

class MainWin (QtGui.QMainWindow):

    def __init__ (self, parent=None):
        QtGui.QMainWindow.__init__ (self, parent)
        self.setupUI ()
        self.setupMenus ()
        self.setupConnections ()

    def setupUI (self) :
        self.setStatusBar (QtGui.QStatusBar ())
        self.resize (640, 480);

        self.split = QtGui.QSplitter ()
        self.setCentralWidget (self.split)

        self.treeView = TreeView ()
        self.split.addWidget (self.treeView)

        self.rightTabs = QtGui.QTabWidget ()
        self.rightTabs.tabBar().setFocusPolicy (QtCore.Qt.NoFocus)
        self.rightTabs.setTabPosition (QtGui.QTabWidget.South)
        self.split.addWidget (self.rightTabs)

        self.editors = { }

    def action (self, name, icon_image = None ) :
        title = name.title ()
        act = QtGui.QAction ("&" + title, self)
        if icon_image :
           act.setIcon (icon_image)
        return act

    def setupMenus (self):

        click = QtCore.SIGNAL ("triggered()")

        fileMenu = self.menuBar().addMenu ("&File")

        act = self.action ("Open...")
        self.connect (act, click, self.openFile)
        fileMenu.addAction (act)

        act = self.action ("Quit")
        self.connect (act, click, self.close)
        fileMenu.addAction (act)

    def setupConnections (self):

        self.connect (self.treeView,
                      QtCore.SIGNAL("itemClicked (QTreeWidgetItem *, int)"),
                      self.treeView_itemClicked)

    def treeView_itemClicked (self, tree_node, column) :
        pass

    def openSource (self, fileName, line, col, stop_line, stop_col):
        if fileName not in self.editors :

           view = QtGui.QTextEdit ()
           name = QtCore.QString  (os.path.basename (fileName))

           self.rightTabs.addTab (view, name)
           self.editors[fileName] = view

           text = open(fileName).read()
           view.setText (text)

        edit = self.editors[fileName]
        self.rightTabs.setCurrentWidget (edit)

        cursor = edit.textCursor()
        cursor.movePosition (QtGui.QTextCursor.Start)
        cursor.movePosition (QtGui.QTextCursor.Down, QtGui.QTextCursor.MoveAnchor, line-1)
        cursor.movePosition (QtGui.QTextCursor.Right, QtGui.QTextCursor.MoveAnchor, col-1)

        if stop_line == line :
           cursor.movePosition (QtGui.QTextCursor.Right, QtGui.QTextCursor.KeepAnchor, stop_col-col)
        else :
           cursor.movePosition (QtGui.QTextCursor.Down, QtGui.QTextCursor.KeepAnchor, stop_line-line)
           cursor.movePosition (QtGui.QTextCursor.StartOfLine, QtGui.QTextCursor.KeepAnchor)
           cursor.movePosition (QtGui.QTextCursor.Right, QtGui.QTextCursor.KeepAnchor, stop_col)
        edit.setTextCursor(cursor)

    def openFile (self) :
        fileName = QtGui.QFileDialog.getOpenFileName (self, "Open File")
        if fileName :
           self.translateFile (str (fileName))

    def translateFile (self, fileName) :
        top = TreeItem (self.treeView, "file " + fileName)
        top.addIcons  ("file-text-32", "text-plain");

    def addBranch (self, above, node, level):
        item = TreeItem (above, txt)

        #if (node.kind == CursorKind.CLASS_DECL or
            #node.kind == CursorKind.STRUCT_DECL or
            #node.kind == CursorKind.UNION_DECL) :
                #item.addIcons ("element-class-16", "code-class")
        #elif (node.kind == CursorKind.FUNCTION_DECL or
              #node.kind == CursorKind.CXX_METHOD or
              #node.kind == CursorKind.CONSTRUCTOR or
              #node.kind == CursorKind.DESTRUCTOR) :
                 #item.addIcons ("element-method-16", "code-function")
        #elif (node.kind == CursorKind.VAR_DECL or
              #node.kind == CursorKind.PARM_DECL or
              #node.kind == CursorKind.FIELD_DECL) :
                 #item.addIcons ("element-variable-16", "code-variable")

        #item.location = node.location
        #item.start = node.extent.start
        #item.stop = node.extent.end

        #item.setToolTip (0, QtCore.QString (str (item.location) + str (item.start) + str (item.stop)))

        #if level > 1 :
           #for t in node.get_children() :
               #self.addBranch (item, t, level-1)

# --------------------------------------------------------------------------

def on_pass_execution(p, fn):
    if p.name == '*warn_function_return':
        global win
        top = TreeItem (win.treeView, "function " + fn.decl.name)
        top.addIcons ("element-method-16", "code-function")
        # For this pass, "fn" will be an instance of gcc.Function:
        print('fn: %r' % fn)
        print('fn.decl.name: %r' % fn.decl.name)

        # fn.decl is an instance of gcc.FunctionDecl:
        print('return type: %r' % str(fn.decl.type.type))
        print('argument types: %r' % [str(t) for t in fn.decl.type.argument_types])
        
    if p.name == '*clean_state':
        global app
        app.exec_ ()

# --------------------------------------------------------------------------

app = QtGui.QApplication ([""])
win = MainWin ()
win.show ()
gcc.register_callback(gcc.PLUGIN_PASS_EXECUTION, on_pass_execution)
# app.exec_ ()
